@@ -8,16 +8,27 @@ import pandas as pd
 import matplotlib.pyplot as plt
 
 from pyiron_base import PyironFactory, DataContainer
-from pyiron_contrib.atomistics.atomicrex.function_factory import FunctionFactory, FunctionParameterList
+from pyiron_contrib.atomistics.atomicrex.function_factory import (
+    FunctionFactory,
+    FunctionParameterList,
+)
 from pyiron_contrib.atomistics.atomicrex.utility_functions import write_pretty_xml
 
+
 class ARPotFactory(PyironFactory):
     """
     Factory class providing convenient acces to fittable potential types.
     TODO: add other potential types supported by atomicrex.
     """
+
     @staticmethod
-    def eam_potential(identifier="EAM", export_file="output.eam.fs", rho_range_factor=2.0, resolution=10000, species=["*","*"]):
+    def eam_potential(
+        identifier="EAM",
+        export_file="output.eam.fs",
+        rho_range_factor=2.0,
+        resolution=10000,
+        species=["*", "*"],
+    ):
         return EAMPotential(
             identifier=identifier,
             export_file=export_file,
@@ -35,36 +46,40 @@ class ARPotFactory(PyironFactory):
         )
 
     @staticmethod
-    def lennard_jones_potential(sigma, epsilon, cutoff, species={"a": "*", "b": "*"}, identifier="LJ"):
-        return LJPotential(sigma, epsilon, cutoff, species=species, identifier=identifier)
+    def lennard_jones_potential(
+        sigma, epsilon, cutoff, species={"a": "*", "b": "*"}, identifier="LJ"
+    ):
+        return LJPotential(
+            sigma, epsilon, cutoff, species=species, identifier=identifier
+        )
 
     @staticmethod
-    def tersoff_potential(elements, param_file=None):
-        return TersoffPotential(elements=elements, param_file=param_file)
+    def tersoff_potential(elements, export_file=None):
+        return TersoffPotential(elements=elements, export_file=export_file)
 
     @staticmethod
-    def abop_potential(elements, param_file=None):
-        return ABOPPotential(elements=elements, param_file=param_file)
+    def abop_potential(elements, export_file=None):
+        return ABOPPotential(elements=elements, export_file=export_file)
 
 
 class AbstractPotential(DataContainer):
-    """Potentials should inherit from this class for hdf5 storage.
-    """    
+    """Potentials should inherit from this class for hdf5 storage."""
+
     def __init__(self, init=None, table_name="potential"):
         super().__init__(init, table_name=table_name)
-    
-    #def copy_final_to_initial_params(self):
+
+    # def copy_final_to_initial_params(self):
     #    raise NotImplementedError("Should be implemented in the subclass.")
-    
+
     def write_xml_file(self, directory):
         """
         Write the potential.xml file for the atomicrex job.
         Has to be defined.
         Args:
             directory (str): working directory path
-        """    
+        """
         raise NotImplementedError("Has to be implemented in the subclass")
-    
+
     def _parse_final_parameters(self, lines):
         raise NotImplementedError("Has to be implemented in the subclass")
 
@@ -77,31 +92,34 @@ class AbstractPotential(DataContainer):
 
         Args:
             job (pyiron_job): Takes the fit job as argument, to obtain f.e. the working directory.
-        """ 
+        """
         raise NotImplementedError("Should be implemented in the subclass")
-    
+
     def _plot_final_potential(self):
         raise NotImplementedError("Should be implemented in the subclass")
 
 
 class BOPAbstract(AbstractPotential):
-    def __init__(self, init=None, elements=None, param_file=None, identifier=None):
+    def __init__(self, init=None, elements=None, export_file=None, identifier=None):
         super().__init__(init=init)
         if init is None:
             self.identifier = identifier
             if elements is not None:
-                if param_file is None:
-                    self.param_file = f"{''.join(elem for elem in elements)}.tersoff"
+                if export_file is None:
+                    self.export = f"{''.join(elem for elem in elements)}.tersoff"
                 else:
-                    self.param_file = param_file
-                
+                    self.export_file = export_file
+
                 self._tag_dict = _get_tag_dict(elements)
                 self.elements = elements
                 if self.identifier == "tersoff":
                     self.parameters = TersoffParameters(self._tag_dict)
                 elif self.identifier == "abop":
                     self.parameters = ABOPParameters(self._tag_dict)
-       
+
+    @property
+    def param_file(self):
+        return self.export_file
 
     def write_xml_file(self, directory):
         self.parameters._write_tersoff_file(directory)
@@ -112,7 +130,7 @@ class BOPAbstract(AbstractPotential):
         tersoff.set("species-b", "*")
 
         output = ET.SubElement(tersoff, "export-potential")
-        output.text = f"{self.param_file}"
+        output.text = f"{self.export_file}"
 
         params = ET.SubElement(tersoff, "param-file")
         params.text = "input.tersoff"
@@ -121,19 +139,18 @@ class BOPAbstract(AbstractPotential):
 
         filename = posixpath.join(directory, "potential.xml")
         write_pretty_xml(tersoff, filename)
-    
-    
+
     def _parse_final_parameters(self, lines):
         for l in lines:
             tag, param, value = _parse_tersoff_line(l)
             self.parameters[tag][param].final_value = value
-    
+
     def _potential_as_pd_df(self, job):
         """
         Makes the tabulated eam potential written by atomicrex usable
         for pyiron lammps jobs.
         """
-        if self.param_file is None:
+        if self.export_file is None:
             raise ValueError("export_file must be set to use the potential with lammps")
 
         species = [el for el in job.input.atom_types.keys()]
@@ -141,27 +158,36 @@ class BOPAbstract(AbstractPotential):
         for s in species:
             species_str += f"{s} "
 
-        pot = pd.DataFrame({
-            "Name": f"{self.identifier}",
-            "Filename": [[f"{job.working_directory}/{self.param_file}"]],
-            'Model': ['Custom'],
-            "Species": [species],
-            "Config": [[
-                "pair_style tersoff\n",
-                f"pair_coeff * * {job.working_directory}/{self.param_file} {species_str}\n",
-                ]]
-        })    
+        pot = pd.DataFrame(
+            {
+                "Name": f"{self.identifier}",
+                "Filename": [[f"{job.working_directory}/{self.export_file}"]],
+                "Model": ["Custom"],
+                "Species": [species],
+                "Config": [
+                    [
+                        "pair_style tersoff\n",
+                        f"pair_coeff * * {job.working_directory}/{self.export_file} {species_str}\n",
+                    ]
+                ],
+            }
+        )
         return pot
 
 
 class TersoffPotential(BOPAbstract):
-    def __init__(self, init=None, elements=None, param_file=None):
-        super().__init__(init=init, elements=elements, param_file=param_file, identifier="tersoff")
+    def __init__(self, init=None, elements=None, export_file=None):
+        super().__init__(
+            init=init, elements=elements, export_file=export_file, identifier="tersoff"
+        )
 
 
 class ABOPPotential(BOPAbstract):
-    def __init__(self, init=None, elements=None, param_file=None):
-        super().__init__(init=init, elements=elements, param_file=param_file, identifier="abop")
+    def __init__(self, init=None, elements=None, export_file=None):
+        super().__init__(
+            init=init, elements=elements, export_file=export_file, identifier="abop"
+        )
+
 
 # tag regular expression
 tag_re = re.compile("[A-Z][a-z]?")
@@ -183,19 +209,46 @@ tersoff_file_params = [
     "R",
     "D",
     "lambda1",
-    "A"
+    "A",
 ]
 
 # Transform tersoff to abop parameters
-def get_r0(lam1, lam2, A, B): return 1/(lam1-lam2)*np.log(lam1*A/(lam2*B))
-def get_D0(lam1, lam2, A, B): return A*(lam1/lam2 - 1)*np.exp(-lam1/(lam1-lam2)*np.log(lam1*A/lam2*B))
-def get_beta(lam1, lam2): return lam1 / np.sqrt(2*lam1/lam2) 
-def get_S(lam1, lam2): return lam1/lam2
+def get_r0(lam1, lam2, A, B):
+    return 1 / (lam1 - lam2) * np.log(lam1 * A / (lam2 * B))
+
+
+def get_D0(lam1, lam2, A, B):
+    return (
+        A
+        * (lam1 / lam2 - 1)
+        * np.exp(-lam1 / (lam1 - lam2) * np.log(lam1 * A / lam2 * B))
+    )
+
+
+def get_beta(lam1, lam2):
+    return lam1 / np.sqrt(2 * lam1 / lam2)
+
+
+def get_S(lam1, lam2):
+    return lam1 / lam2
+
+
 # abop to tersoff parameters
-def get_lam1(beta, S): return beta*np.sqrt(2*S)
-def get_lam2(beta, S): return beta*np.sqrt(2/S)
-def get_A(beta, S, D0, r0): return D0/(S-1)*np.exp(beta*np.sqrt(2*S)*r0)
-def get_B(beta, S, D0, r0): return D0*S/(S-1)*np.exp(beta*np.sqrt(2/S)*r0)
+def get_lam1(beta, S):
+    return beta * np.sqrt(2 * S)
+
+
+def get_lam2(beta, S):
+    return beta * np.sqrt(2 / S)
+
+
+def get_A(beta, S, D0, r0):
+    return D0 / (S - 1) * np.exp(beta * np.sqrt(2 * S) * r0)
+
+
+def get_B(beta, S, D0, r0):
+    return D0 * S / (S - 1) * np.exp(beta * np.sqrt(2 / S) * r0)
+
 
 class BOPParameters(DataContainer):
     def __init__(self, tag_dict=None):
@@ -217,9 +270,10 @@ class BOPParameters(DataContainer):
 
         Returns:
             list[str]: list of tags f.e. ["SiSiSi", "SiSiC", ...]
-        """        
+        """
         return _tag_list(elements)
 
+
 class TersoffParameters(BOPParameters):
     def __init__(self, tag_dict=None):
         super().__init__(tag_dict)
@@ -232,7 +286,7 @@ class TersoffParameters(BOPParameters):
             # If this order is kept always? this allows to iterate through it in a for loop
             # in some sitauations, instead of looking up the keys
             # 3 body params m(not fittable), gamma, lambda3, c, d, theta0
-            self[tag].add_parameter("m", start_val=3, fitable=False) # 3 or 1
+            self[tag].add_parameter("m", start_val=3, fitable=False)  # 3 or 1
 
             self[tag].add_parameter("gamma", start_val=None, tag=tag)
             self[tag].add_parameter("lambda3", start_val=None, tag=tag)
@@ -244,7 +298,7 @@ class TersoffParameters(BOPParameters):
             self[tag].add_parameter("n", start_val=None, fitable=False)
             self[tag].add_parameter("beta", start_val=None, fitable=v, tag=tag)
             self[tag].add_parameter("lambda2", start_val=None, fitable=v, tag=tag)
-            self[tag].add_parameter("B", start_val=None, fitable=v, tag=tag)           
+            self[tag].add_parameter("B", start_val=None, fitable=v, tag=tag)
             # 2 and 3 body params D and R
             self[tag].add_parameter("D", start_val=None, fitable=False)
             self[tag].add_parameter("R", start_val=None, fitable=False)
@@ -252,7 +306,6 @@ class TersoffParameters(BOPParameters):
             self[tag].add_parameter("lambda1", start_val=None, fitable=v, tag=tag)
             self[tag].add_parameter("A", start_val=None, fitable=v, tag=tag)
 
-            
     def start_vals_from_file(self, file, tag_list=None):
         with open(file) as f:
             for l in f:
@@ -275,15 +328,17 @@ class TersoffParameters(BOPParameters):
                             self[tag][param].start_val = float(l[i])
                         except:
                             KeyError(
-                                f"Could not assign start value of tag: {tag} to parameter: {param}" 
+                                f"Could not assign start value of tag: {tag} to parameter: {param}"
                             )
-    
+
     def _write_tersoff_file(self, directory, filename="input.tersoff"):
         filepath = posixpath.join(directory, filename)
         with open(filepath, "w") as f:
-            f.write("# Tersoff parameter file in lammps/tersoff format written via pyiron atomicrex interface.\n")
+            f.write(
+                "# Tersoff parameter file in lammps/tersoff format written via pyiron atomicrex interface.\n"
+            )
             f.write("# Necessary to fit tersoff and abop potentials.\n#\n")
-            f.write(f"# {' '.join(tersoff_file_params)}\n#\n") 
+            f.write(f"# {' '.join(tersoff_file_params)}\n#\n")
             for tag in self.keys():
                 for el in tag_re.findall(tag):
                     f.write(f"{el} ")
@@ -291,7 +346,7 @@ class TersoffParameters(BOPParameters):
                     param = tersoff_file_params[i]
                     f.write(f"{self[tag][param].start_val} ")
                 f.write("\n")
-    
+
     def _to_xml_element(self):
         fit_dof = ET.Element("fit-dof")
         for tag, parameters in self.items():
@@ -309,28 +364,32 @@ class ABOPParameters(BOPParameters):
         for tag, v in tag_dict.items():
             self[tag] = FunctionParameterList()
 
-            #2 Body parameters, fitable only if v (see https://lammps.sandia.gov/doc/pair_tersoff.html)
+            # 2 Body parameters, fitable only if v (see https://lammps.sandia.gov/doc/pair_tersoff.html)
             self[tag].add_parameter("D0", start_val=None, fitable=v, tag=tag)
             self[tag].add_parameter("S", start_val=None, fitable=v, tag=tag)
             self[tag].add_parameter("r0", start_val=None, fitable=v, tag=tag)
             self[tag].add_parameter("beta", start_val=None, fitable=v, tag=tag)
             self[tag].add_parameter("beta2", start_val=None, fitable=v, tag=tag)
             # default enable False for n in 1/2n term and standard value 1
-            self[tag].add_parameter("powern", start_val=1, fitable=v, enabled=False, tag=tag)
+            self[tag].add_parameter(
+                "powern", start_val=1, fitable=v, enabled=False, tag=tag
+            )
             # 3 Body parameters
             self[tag].add_parameter("gamma", start_val=None, tag=tag)
             self[tag].add_parameter("c", start_val=None, tag=tag)
             self[tag].add_parameter("d", start_val=None, tag=tag)
             self[tag].add_parameter("h", start_val=None, tag=tag)
-            #twomu = lambda3?
-            self[tag].add_parameter("twomu", start_val=None, tag=tag)           
+            # twomu = lambda3?
+            self[tag].add_parameter("twomu", start_val=None, tag=tag)
             # m is not fittable
-            self[tag].add_parameter("m", start_val=1.0, fitable=False, tag=tag) # 3 or 1
+            self[tag].add_parameter(
+                "m", start_val=1.0, fitable=False, tag=tag
+            )  # 3 or 1
             # 2 and 3 body
             # default enable False for cutoff parameters
             self[tag].add_parameter("bigd", start_val=None, enabled=False, tag=tag)
             self[tag].add_parameter("bigr", start_val=None, enabled=False, tag=tag)
-                    
+
     def start_vals_from_file(self, file, tag_list=None):
         if tag_list is None:
             tag_list = self.list_groups()
@@ -338,34 +397,34 @@ class ABOPParameters(BOPParameters):
         t_params = TersoffParameters(tag_dict)
         t_params.start_vals_from_file(file, tag_list=tag_list)
         for tag, params in t_params.items():
-                lam1 = params["lambda1"].start_val
-                lam2 = params["lambda2"].start_val
-                A = params["A"].start_val
-                B = params["B"].start_val
-                
-                self[tag]["h"].start_val = -params["theta0"].start_val
+            lam1 = params["lambda1"].start_val
+            lam2 = params["lambda2"].start_val
+            A = params["A"].start_val
+            B = params["B"].start_val
 
-                if tag_dict[tag]:
-                    self[tag]["r0"].start_val = get_r0(lam1, lam2, A, B)
-                    self[tag]["D0"].start_val = get_D0(lam1, lam2, A, B)
-                    self[tag]["S"].start_val = get_S(lam1, lam2)
-                    self[tag]["beta"].start_val = get_beta(lam1, lam2)
-                else: # pair only parameters are not read for these tags and are set to 0 to prevent 0 division errors
-                    self[tag]["r0"].start_val = 0.0
-                    self[tag]["D0"].start_val = 0.0
-                    self[tag]["S"].start_val = 0.0
-                    self[tag]["beta"].start_val = 0.0
+            self[tag]["h"].start_val = -params["theta0"].start_val
 
-                # Other parameters
-                self[tag]["m"].start_val = params["m"].start_val
-                self[tag]["powern"].start_val = params["n"].start_val
-                self[tag]["c"].start_val = params["c"].start_val
-                self[tag]["d"].start_val = params["d"].start_val
-                self[tag]["bigd"].start_val = params["D"].start_val
-                self[tag]["bigr"].start_val = params["R"].start_val
-                self[tag]["gamma"].start_val = params["gamma"].start_val
-                self[tag]["twomu"].start_val = params["lambda3"].start_val
-                self[tag]["beta2"].start_val = params["beta"].start_val
+            if tag_dict[tag]:
+                self[tag]["r0"].start_val = get_r0(lam1, lam2, A, B)
+                self[tag]["D0"].start_val = get_D0(lam1, lam2, A, B)
+                self[tag]["S"].start_val = get_S(lam1, lam2)
+                self[tag]["beta"].start_val = get_beta(lam1, lam2)
+            else:  # pair only parameters are not read for these tags and are set to 0 to prevent 0 division errors
+                self[tag]["r0"].start_val = 0.0
+                self[tag]["D0"].start_val = 0.0
+                self[tag]["S"].start_val = 0.0
+                self[tag]["beta"].start_val = 0.0
+
+            # Other parameters
+            self[tag]["m"].start_val = params["m"].start_val
+            self[tag]["powern"].start_val = params["n"].start_val
+            self[tag]["c"].start_val = params["c"].start_val
+            self[tag]["d"].start_val = params["d"].start_val
+            self[tag]["bigd"].start_val = params["D"].start_val
+            self[tag]["bigr"].start_val = params["R"].start_val
+            self[tag]["gamma"].start_val = params["gamma"].start_val
+            self[tag]["twomu"].start_val = params["lambda3"].start_val
+            self[tag]["beta2"].start_val = params["beta"].start_val
 
     def _write_tersoff_file(self, directory, filename="input.tersoff"):
         tersoff = abop_to_tersoff_params(self)
@@ -392,7 +451,7 @@ def abop_to_tersoff_params(abop_parameters):
 
     tag_dict = _tagdict_from_taglist(abop_parameters.groups())
     tersoff = TersoffParameters(tag_dict)
-     
+
     for tag, params in tersoff.items():
         if tag_dict[tag]:
             beta = abop_parameters[tag]["beta"].start_val
@@ -405,7 +464,7 @@ def abop_to_tersoff_params(abop_parameters):
             params["B"].start_val = get_B(beta, S, D0, r0)
             params["lambda1"].start_val = get_lam1(beta, S)
             params["lambda2"].start_val = get_lam2(beta, S)
-        else: # pair only parameters are not read for these tags and are set to 0 to prevent 0 division errors
+        else:  # pair only parameters are not read for these tags and are set to 0 to prevent 0 division errors
             params["A"].start_val = 0.0
             params["B"].start_val = 0.0
             params["lambda1"].start_val = 0.0
@@ -432,8 +491,17 @@ class LJPotential(AbstractPotential):
     mostly implemented for testing and demonstatration purposes.
     TODO: Check if this still works after alls changes
     TODO: Implement missing functionality.
-    """    
-    def __init__(self, init=None, sigma=None, epsilon=None, cutoff=None, species=None, identifier=None):
+    """
+
+    def __init__(
+        self,
+        init=None,
+        sigma=None,
+        epsilon=None,
+        cutoff=None,
+        species=None,
+        identifier=None,
+    ):
         super().__init__(init=init)
         if init is None:
             self.sigma = sigma
@@ -445,7 +513,7 @@ class LJPotential(AbstractPotential):
     def write_xml_file(self, directory):
         """
         Internal function to create xml element.
-        """        
+        """
         lj = ET.Element("lennard-jones")
         lj.set("id", f"{self.identifier}")
         lj.set("species-a", self.species["a"])
@@ -465,20 +533,21 @@ class LJPotential(AbstractPotential):
         filename = posixpath.join(directory, "potential.xml")
         write_pretty_xml(lj, filename)
 
+
 class EAMlikeMixin:
-    def copy_final_to_initial_params(self):
+    def copy_final_to_initial_params(self, filter_func=None):
         """
         Copies final values of function paramters to start values.
         This f.e. allows to continue global with local minimization.
-        """        
+        """
         for functions in self._function_tuple:
             for f in functions.values():
-                f.copy_final_to_initial_params()
+                f.copy_final_to_initial_params(filter_func=filter_func)
 
-    def lock_parameters(self):
+    def lock_parameters(self, filter_func=None):
         for functions in self._function_tuple:
             for f in functions.values():
-                f.lock_parameters()
+                f.lock_parameters(filter_func=filter_func)
 
     def count_parameters(self, enabled_only=True):
         parameters = 0
@@ -487,6 +556,7 @@ class EAMlikeMixin:
                 parameters += f.count_parameters(enabled_only=enabled_only)
         return parameters
 
+
 class EAMPotential(AbstractPotential, EAMlikeMixin):
     """
     Embedded-Atom-Method potential.
@@ -498,7 +568,7 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
     Example:
     eam.pair_interactions["V"] = morse_function
 
-    """    
+    """
 
     def __init__(
         self,
@@ -507,8 +577,9 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
         export_file=None,
         rho_range_factor=None,
         resolution=None,
-        species=None):
-        
+        species=None,
+    ):
+
         super().__init__(init=init)
         if init is None:
             self.pair_interactions = DataContainer(table_name="pair_interactions")
@@ -519,10 +590,14 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
             self.rho_range_factor = rho_range_factor
             self.resolution = resolution
             self.species = species
-    
+
     @property
     def _function_tuple(self):
-        return (self.pair_interactions, self.electron_densities, self.embedding_energies)
+        return (
+            self.pair_interactions,
+            self.electron_densities,
+            self.embedding_energies,
+        )
 
     def _potential_as_pd_df(self, job):
         """
@@ -537,22 +612,26 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
         for s in species:
             species_str += f"{s} "
 
-        pot = pd.DataFrame({
-            "Name": f"{self.identifier}",
-            "Filename": [[f"{job.working_directory}/{self.export_file}"]],
-            'Model': ['Custom'],
-            "Species": [species],
-            "Config": [[
-                "pair_style eam/fs\n",
-                f"pair_coeff * * {job.working_directory}/{self.export_file} {species_str}\n",
-                ]]
-        })    
+        pot = pd.DataFrame(
+            {
+                "Name": f"{self.identifier}",
+                "Filename": [[f"{job.working_directory}/{self.export_file}"]],
+                "Model": ["Custom"],
+                "Species": [species],
+                "Config": [
+                    [
+                        "pair_style eam/fs\n",
+                        f"pair_coeff * * {job.working_directory}/{self.export_file} {species_str}\n",
+                    ]
+                ],
+            }
+        )
         return pot
 
     def write_xml_file(self, directory):
         """
         Internal function to convert to an xml element
-        """        
+        """
         eam = ET.Element("eam")
         eam.set("id", f"{self.identifier}")
         eam.set("species-a", f"{self.species[0]}")
@@ -600,15 +679,21 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
 
         Raises:
             KeyError: Raises if a parsed parameter can't be matched to a function.
-        """        
+        """
         for l in lines:
             identifier, leftover, value = _parse_parameter_line(l)
             if identifier in self.pair_interactions:
-                self.pair_interactions[identifier]._parse_final_parameter(leftover, value)
+                self.pair_interactions[identifier]._parse_final_parameter(
+                    leftover, value
+                )
             elif identifier in self.electron_densities:
-                self.electron_densities[identifier]._parse_final_parameter(leftover, value)
+                self.electron_densities[identifier]._parse_final_parameter(
+                    leftover, value
+                )
             elif identifier in self.embedding_energies:
-                self.embedding_energies[identifier]._parse_final_parameter(leftover, value)
+                self.embedding_energies[identifier]._parse_final_parameter(
+                    leftover, value
+                )
             else:
 
                 raise KeyError(
@@ -616,7 +701,6 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
                     "Fitting parameters of screening functions probably doesn't work right now"
                 )
 
-
     def plot_final_potential(self, job, filename=None):
         """
         Plots the the fitted potential. Reads the necessary data from eam.fs file (funcfl format used by lammps),
@@ -626,79 +710,61 @@ class EAMPotential(AbstractPotential, EAMlikeMixin):
         Args:
             job (AtomicrexJob): Instance of the job to construct filename.
             filename (str, optional): If a filename is given this eam is plotted instead of the fitted one. Defaults to None.
-        """        
-
-
-        # Read the cotents of the eam file
-        # this is copy pasted from one of my old scripts and could probably be reworked
-        elements = {}
-        
+        """
         if filename is None:
             filename = f"{job.working_directory}/{self.export_file}"
 
-        with open(filename, "r") as f:
-            # skip comment lines
-            for _ in range(3):
-                f.readline()
-            
-            element_list =  f.readline().split()
-            for element in element_list[1:]:
-                elements[element] = {}
-            
-            Nrho, drho, Nr, dr, cutoff = f.readline().split()
-            Nrho = int(Nrho)
-            drho = float(drho)
-            Nr = int(Nr)
-            dr = float(dr)
-            cutoff = float(cutoff)
+        elements, r_values, rho_values = read_eam_fs_file(filename=filename)
 
-            rho_values = np.linspace(0,Nrho*drho, Nrho, endpoint=False)
-            r_values = np.linspace(0, Nr*dr, Nr, endpoint=False)
-
-            for element in elements:
-                # skip a line with unnecessary information
-                f.readline()
-                elements[element]["F"] = np.fromfile(f, count=Nrho, sep=" ")
-                for rho_element in elements:
-                    elements[element]["rho_{}{}".format(element, rho_element)] = np.fromfile(f, count = Nr, sep=" ")
-
-            # V_ij = V_ji so it is written only once in the file => avoid attempts to read it twice
-            # with a list of elements where it has been read
-            # TODO: Have another look how to do this checking
-            V_written = []
-            for element in elements:
-                for V_element in elements:
-                    elementV_element = "{}{}".format(element, V_element)
-                    V_elementelement = "{}{}".format(V_element, element)
-                    if not elementV_element in V_written and not V_elementelement in V_written:
-                        elements[element]["V_{}".format(elementV_element)] = np.fromfile(f, count = Nr, sep=" ")
-                        # The tabulated values are not V(r) but V(r) * r, so they are divided by r here,
-                        # with exception of the first value to prevent division by 0.
-                        elements[element]["V_{}".format(elementV_element)][1:] = elements[element]["V_{}".format(elementV_element)][1:] / r_values[1:]
-                        V_written.append(elementV_element)
-        
         # TODO: figure out how to index ax for multiple elements
-        fig, ax = plt.subplots(nrows=3*len(elements), ncols=len(elements), figsize=(len(elements)*8, len(elements)*3*6), squeeze=False)
+        fig, ax = plt.subplots(
+            nrows=3 * len(elements),
+            ncols=len(elements),
+            figsize=(len(elements) * 8, len(elements) * 3 * 6),
+            squeeze=False,
+        )
         for i, (el, pot_dict) in enumerate(elements.items()):
             for j, (pot, y) in enumerate(pot_dict.items()):
                 if pot == "F":
                     xdata = rho_values
                     xlabel = "$\\rho $ [a.u.]"
                     k = 0
+                    ylim = (-5, 5)
                 elif "rho" in pot:
                     xdata = r_values
                     xlabel = "r [$\AA$]"
                     k = 1
+                    ylim = (-0.2, 1)
                 elif "V" in pot:
                     xdata = r_values[1:]
                     y = y[1:]
                     xlabel = "r [$\AA$]"
                     k = 2
-
-                ax[i+k, 0].plot(xdata, y)
-                ax[i+k, 0].set(ylim=(-5,5), title=f"{el} {pot}", xlabel=xlabel)
+                    ylim = (-2, 2)
+                ax[i + k, 0].plot(xdata, y)
+                ax[i + k, 0].set(ylim=ylim, title=f"{el} {pot}", xlabel=xlabel)
         return fig, ax
 
+    def count_local_extrema(
+        self, job, filename=None, count_minima=True, count_maxima=False
+    ):
+        if filename is None:
+            filename = f"{job.working_directory}/{self.export_file}"
+        elements, _r_values, _rho_values = read_eam_fs_file(filename=filename)
+        extrema_dict = {}
+        for el, func in elements.items():
+            extrema_dict[el] = {}
+            for func_name, a in func:
+                extrema = 0
+                if count_minima:
+                    min_arr = np.r_[True, a[1:] < a[:-1]] & np.r_[a[:-1] < a[1:], True]
+                    extrema += len(min_arr[min_arr])
+                if count_maxima:
+                    max_arr = np.r_[True, a[1:] > a[:-1]] & np.r_[a[:-1] > a[1:], True]
+                    extrema += len(max_arr[max_arr])
+                extrema_dict[el][func_name] = extrema
+        return extrema_dict
+
 
 class MEAMPotential(AbstractPotential, EAMlikeMixin):
     def __init__(self, init=None, identifier=None, export_file=None, species=None):
@@ -716,17 +782,17 @@ class MEAMPotential(AbstractPotential, EAMlikeMixin):
     @property
     def _function_tuple(self):
         return (
-                self.pair_interactions,
-                self.electron_densities,
-                self.embedding_energies,
-                self.f_functions,
-                self.g_functions
-            )
+            self.pair_interactions,
+            self.electron_densities,
+            self.embedding_energies,
+            self.f_functions,
+            self.g_functions,
+        )
 
     def write_xml_file(self, directory):
         """
         Internal function to convert to an xml element
-        """        
+        """
         meam = ET.Element("meam")
         meam.set("id", f"{self.identifier}")
         meam.set("species-a", f"{self.species[0]}")
@@ -759,7 +825,7 @@ class MEAMPotential(AbstractPotential, EAMlikeMixin):
             f_function.set("species-b", f"{pot.species[1]}")
             f_function.set("function", f"{pot.identifier}")
             functions.append(pot._to_xml_element())
-        
+
         for pot in self.g_functions.values():
             g_function = ET.SubElement(mapping, "g-function")
             g_function.set("species-a", f"{pot.species[0]}")
@@ -777,7 +843,6 @@ class MEAMPotential(AbstractPotential, EAMlikeMixin):
         filename = posixpath.join(directory, "potential.xml")
         write_pretty_xml(meam, filename)
 
-
     def _parse_final_parameters(self, lines):
         """
         Internal Function.
@@ -788,19 +853,25 @@ class MEAMPotential(AbstractPotential, EAMlikeMixin):
 
         Raises:
             KeyError: Raises if a parsed parameter can't be matched to a function.
-        """        
+        """
         for l in lines:
-            identifier, param, value = _parse_parameter_line(l)
+            identifier, leftover, value = _parse_parameter_line(l)
             if identifier in self.pair_interactions:
-                self.pair_interactions[identifier]._parse_final_parameter(leftover, value)
+                self.pair_interactions[identifier]._parse_final_parameter(
+                    leftover, value
+                )
             elif identifier in self.electron_densities:
-                self.electron_densities[identifier]._parse_final_parameter(leftover, value)
+                self.electron_densities[identifier]._parse_final_parameter(
+                    leftover, value
+                )
             elif identifier in self.f_functions:
                 self.f_functions[identifier]._parse_final_parameter(leftover, value)
             elif identifier in self.g_functions:
                 self.g_functions[identifier]._parse_final_parameter(leftover, value)
             elif identifier in self.embedding_energies:
-                self.embedding_energies[identifier]._parse_final_parameter(leftover, value)
+                self.embedding_energies[identifier]._parse_final_parameter(
+                    leftover, value
+                )
             else:
                 raise KeyError(
                     f"Can't find {identifier} in potential, probably something went wrong during parsing.\n"
@@ -825,11 +896,11 @@ def _parse_parameter_line(line):
     This allows to flexibly parse the lines for the different types
     of functional forms by passing leftover to a function defined
     in the corresponding function class.
-    
+
     TODO: Add parsing of polynomial parameters.
     Returns:
         (str, [str], float): identifier, leftover, value
-    """        
+    """
     line = line.strip().split()
     value = float(line[1])
     info = line[0].split("].")
@@ -851,6 +922,7 @@ def _tag_list(elements):
     """
     return ["".join(el) for el in itertools.product(elements, repeat=3)]
 
+
 def _get_tag_dict(elements):
     """
     Internal helpfer function for Tersoff and ABOP potentials.
@@ -870,18 +942,19 @@ def _get_tag_dict(elements):
         list[str]: list of elements
 
     Returns:
-        dict[dict]: elemen tags and 
+        dict[dict]: elemen tags and
     """
     tags = {}
     tag_list = list(itertools.product(elements, repeat=3))
     for l in tag_list:
         k = "".join(l)
         if l[1] == l[2]:
-            tags[k] = True # True if 2 and 3 body
+            tags[k] = True  # True if 2 and 3 body
         else:
-            tags[k] = False # False if only 3 body
+            tags[k] = False  # False if only 3 body
     return tags
 
+
 def _parse_tersoff_line(line):
     """
     Internal Function.
@@ -889,7 +962,7 @@ def _parse_tersoff_line(line):
     from an atomicrex output line looking like:
 
     tersoff[Tersoff].lambda1[SiSiSi]: 2.4799 [0:]
-    
+
     Returns:
         [(str, str, float): tag, param, value
     """
@@ -901,6 +974,7 @@ def _parse_tersoff_line(line):
     tag = info[1].rstrip("]:")
     return tag, param, value
 
+
 def _tagdict_from_taglist(taglist):
     tagdict = {}
     for tag in taglist:
@@ -909,4 +983,61 @@ def _tagdict_from_taglist(taglist):
             tagdict[tag] = True
         else:
             tagdict[tag] = False
-    return tagdict
\ No newline at end of file
+    return tagdict
+
+
+def read_eam_fs_file(filename):
+    # Read the cotents of an eam/fs file and returns it as a dicitionary of numpy arrays
+    # this is copy pasted from one of my old scripts and could probably be reworked
+    elements = {}
+    with open(filename, "r") as f:
+        # skip comment lines
+        for _ in range(3):
+            f.readline()
+
+        element_list = f.readline().split()
+        for element in element_list[1:]:
+            elements[element] = {}
+
+        Nrho, drho, Nr, dr, cutoff = f.readline().split()
+        Nrho = int(Nrho)
+        drho = float(drho)
+        Nr = int(Nr)
+        dr = float(dr)
+        cutoff = float(cutoff)
+
+        rho_values = np.linspace(0, Nrho * drho, Nrho, endpoint=False)
+        r_values = np.linspace(0, Nr * dr, Nr, endpoint=False)
+
+        for element in elements:
+            # skip a line with unnecessary information
+            f.readline()
+            elements[element]["F"] = np.fromfile(f, count=Nrho, sep=" ")
+            for rho_element in elements:
+                elements[element][
+                    "rho_{}{}".format(element, rho_element)
+                ] = np.fromfile(f, count=Nr, sep=" ")
+
+        # V_ij = V_ji so it is written only once in the file => avoid attempts to read it twice
+        # with a list of elements where it has been read
+        # TODO: Have another look how to do this checking
+        V_written = []
+        for element in elements:
+            for V_element in elements:
+                elementV_element = "{}{}".format(element, V_element)
+                V_elementelement = "{}{}".format(V_element, element)
+                if (
+                    not elementV_element in V_written
+                    and not V_elementelement in V_written
+                ):
+                    elements[element]["V_{}".format(elementV_element)] = np.fromfile(
+                        f, count=Nr, sep=" "
+                    )
+                    # The tabulated values are not V(r) but V(r) * r, so they are divided by r here,
+                    # with exception of the first value to prevent division by 0.
+                    elements[element]["V_{}".format(elementV_element)][1:] = (
+                        elements[element]["V_{}".format(elementV_element)][1:]
+                        / r_values[1:]
+                    )
+                    V_written.append(elementV_element)
+    return elements, r_values, rho_values
