@@ -10,9 +10,11 @@ import os
 import pandas as pd
 import posixpath
 import scipy.constants
-from pyiron_base import Settings, GenericParameters, GenericJob, Executable
-from pyiron_atomistics import ase_to_pyiron
+from pyiron_base import state, GenericParameters, GenericJob, Executable, FlattenedStorage
+from pyiron_atomistics import ase_to_pyiron, Atoms
+from pyiron_atomistics.atomistics.structure.structurestorage import StructureStorage
 from pyiron_contrib.atomistics.mlip.cfgs import savecfgs, loadcfgs, Cfg
+from pyiron_contrib.atomistics.mlip.potential import MtpPotential
 
 __author__ = "Jan Janssen"
 __copyright__ = "Copyright 2020, Max-Planck-Institut für Eisenforschung GmbH - " \
@@ -23,8 +25,6 @@ __email__ = "janssen@mpie.de"
 __status__ = "development"
 __date__ = "Sep 1, 2017"
 
-s = Settings()
-
 
 gpa_to_ev_ang = 1e22 / scipy.constants.physical_constants['joule-electron volt relationship'][0]
 
@@ -32,12 +32,13 @@ gpa_to_ev_ang = 1e22 / scipy.constants.physical_constants['joule-electron volt r
 class Mlip(GenericJob):
     def __init__(self, project, job_name):
         super(Mlip, self).__init__(project, job_name)
-        self.__version__ = None
+        self.__version__ = '0.1.0'
         self.__name__ = "Mlip"
         self._executable_activate()
         self._job_dict = {}
         self.input = MlipParameter()
         self._command_line = CommandLine()
+        self._potential = MtpPotential()
 
     def _executable_activate(self, enforce=False):
         if self._executable is None or enforce:
@@ -45,11 +46,11 @@ class Mlip(GenericJob):
                 self._executable = Executable(
                     codename=self.__name__,
                     module=self.__module__.split(".")[-2],
-                    path_binary_codes=s.resource_paths,
+                    path_binary_codes=state.settings.resource_paths,
                 )
             else:
                 self._executable = Executable(
-                    codename=self.__name__, path_binary_codes=s.resource_paths
+                    codename=self.__name__, path_binary_codes=state.settings.resource_paths
                 )
 
     @property
@@ -96,6 +97,13 @@ class Mlip(GenericJob):
                         ]]
             })
 
+    @property
+    def potential(self):
+        if self.status.finished:
+            return self._potential
+        else:
+            raise ValueError("potential only available on successfully finished jobs")
+
     def set_input_to_read_only(self):
         """
         This function enforces read-only mode for the input classes, but it has to be implement in the individual
@@ -135,6 +143,21 @@ class Mlip(GenericJob):
         self._command_line[0] = self._command_line[0].replace('iteration_auto', str(int(self.input['iteration'])))
         self._command_line.write_file(file_name='mlip.sh', cwd=self.working_directory)
 
+        species = np.array(species)
+        input_store = StructureStorage()
+        input_store.add_array('energy', dtype=np.float64, shape=(1,), per='chunk')
+        input_store.add_array('forces', dtype=np.float64, shape=(3,), per='element')
+        input_store.add_array('stress', dtype=np.float64, shape=(6,), per='chunk')
+        input_store.add_array('grade',  dtype=np.float64, shape=(1,), per='chunk')
+        for cfg in loadcfgs(os.path.join(self.working_directory, "training.cfg")):
+            struct = Atoms(symbols=species[np.cast[np.int64](cfg.types)], positions=cfg.pos, cell=cfg.lat, pbc=[True]*3) # HACK for pbc
+            input_store.add_structure(struct, identifier=cfg.desc,
+                                      energy=cfg.energy, forces=cfg.forces, stress=cfg.stresses
+            )
+
+        with self.project_hdf5.open('input') as hdf5_input:
+            input_store.to_hdf(hdf=hdf5_input, group_name="training_data")
+
     def collect_logfiles(self):
         pass
 
@@ -154,6 +177,28 @@ class Mlip(GenericJob):
             _, _, _, _, _, _, grades_lst, job_id_grades_lst, timestep_grades_lst = read_cgfs(file_name)
         else:
             grades_lst, job_id_grades_lst, timestep_grades_lst = [], [], []
+        self._potential.load(os.path.join(self.working_directory, "Trained.mtp_"))
+
+        training_store = FlattenedStorage()
+        training_store.add_array('energy', dtype=np.float64, shape=(1,), per='chunk')
+        training_store.add_array('forces', dtype=np.float64, shape=(3,), per='element')
+        training_store.add_array('stress', dtype=np.float64, shape=(6,), per='chunk')
+        training_store.add_array('grade',  dtype=np.float64, shape=(1,), per='chunk')
+        for cfg in loadcfgs(os.path.join(self.working_directory, "training_efs.cfg")):
+            training_store.add_chunk(len(cfg.pos), identifier=cfg.desc,
+                    energy=cfg.energy, forces=cfg.forces, stress=cfg.stresses, grade=cfg.grade
+            )
+
+        testing_store = FlattenedStorage()
+        testing_store.add_array('energy', dtype=np.float64, shape=(1,), per='chunk')
+        testing_store.add_array('forces', dtype=np.float64, shape=(3,), per='element')
+        testing_store.add_array('stress', dtype=np.float64, shape=(6,), per='chunk')
+        testing_store.add_array('grade',  dtype=np.float64, shape=(1,), per='chunk')
+        for cfg in loadcfgs(os.path.join(self.working_directory, "testing_efs.cfg")):
+            testing_store.add_chunk(len(cfg.pos), identifier=cfg.desc,
+                    energy=cfg.energy, forces=cfg.forces, stress=cfg.stresses, grade=cfg.grade
+            )
+
         with self.project_hdf5.open('output') as hdf5_output:
             hdf5_output['grades'] = grades_lst
             hdf5_output['job_id'] = job_id_grades_lst
@@ -162,6 +207,9 @@ class Mlip(GenericJob):
             hdf5_output['timestep_diff'] = timestep_diff_lst
             hdf5_output['job_id_new'] = job_id_new_training_lst
             hdf5_output['timestep_new'] = timestep_new_training_lst
+            self._potential.to_hdf(hdf=hdf5_output)
+            training_store.to_hdf(hdf=hdf5_output, group_name="training_efs")
+            testing_store.to_hdf(hdf=hdf5_output, group_name="testing_efs")
 
     def get_structure(self, iteration_step=-1):
         job = self.project.load(self['output/job_id_diff'][iteration_step])
@@ -193,6 +241,10 @@ class Mlip(GenericJob):
                     time_step_end=end,
                     time_step_delta=delta
                 )
+        if self.status.finished:
+            with self._hdf5.open("output") as hdf_output:
+                if "potential" in hdf_output.list_groups():
+                    self._potential.from_hdf(hdf=hdf_output)
 
     def get_suggested_number_of_configuration(self, species_count=None, multiplication_factor=2.0):
         if self.input['filepath'] == 'auto':
@@ -291,16 +343,20 @@ class Mlip(GenericJob):
             if ham.__name__ == "TrainingContainer":
                 job = ham.to_object()
                 all_species.update(job.get_elements())
-                pd = job.to_pandas()
-                for time_step, (name, atoms, energy, forces, _) in islice(enumerate(pd.itertuples(index=False)),
-                                                                          start, end, delta):
-                    atoms = ase_to_pyiron(atoms)
-                    indices_lst.append(atoms.indices)
-                    position_lst.append(atoms.positions)
-                    forces_lst.append(forces)
-                    cell_lst.append(atoms.cell)
-                    energy_lst.append(energy)
-                    track_lst.append(str(ham.job_id) + "_" + str(time_step))
+                symbol_map = {s: i for i, s in enumerate(sorted(set(all_species)))}
+                if end is None:
+                    end = job.number_of_structures
+                for time_step in range(start, end, delta):
+                    symbols = job._container.get_array("symbols", time_step)
+                    indices_lst.append([symbol_map[s] for s in symbols])
+                    position_lst.append(job._container.get_array("positions", time_step))
+                    forces_lst.append(job._container.get_array("forces", time_step))
+                    cell_lst.append(job._container.get_array("cell", time_step))
+                    energy_lst.append(job._container.get_array("energy", time_step))
+                    if job._container.has_array("stress"):
+                        volume = np.abs(np.linalg.det(cell_lst[-1]))
+                        stress_lst.append(job._container.get_array("stress", time_step) * volume)
+                    track_lst.append(str(ham.job_id) + '_' + str(time_step))
                 continue
             original_dict = {el: ind for ind, el in enumerate(sorted(ham['input/structure/species']))}
             species_dict = {ind: original_dict[el] for ind, el in enumerate(ham['input/structure/species'])}
@@ -394,7 +450,7 @@ class Mlip(GenericJob):
 
     @staticmethod
     def _find_potential(potential_name):
-        for resource_path in s.resource_paths:
+        for resource_path in state.settings.resource_paths:
             if os.path.exists(os.path.join(resource_path, 'mlip', 'potentials', 'templates')):
                 resource_path = os.path.join(resource_path, 'mlip', 'potentials', 'templates')
             if 'potentials' in resource_path and \
@@ -442,6 +498,10 @@ $MLP_COMMAND_PARALLEL train --energy-weight=energy_auto --force-weight=force_aut
             file_content = '''\
 $MLP_COMMAND_PARALLEL train --energy-weight=energy_auto --force-weight=force_auto --stress-weight=stress_auto --max-iter=iteration_auto start.mtp training.cfg > training.log
 $MLP_COMMAND_SERIAL calc-grade Trained.mtp_ training.cfg testing.cfg grades.cfg --mvs-filename=state.mvs > grading.log
+$MLP_COMMAND_SERIAL calc-errors Trained.mtp_ training.cfg > training.errors
+$MLP_COMMAND_SERIAL calc-errors Trained.mtp_ testing.cfg > testing.errors
+$MLP_COMMAND_SERIAL calc-efs Trained.mtp_ training.cfg training_efs.cfg
+$MLP_COMMAND_SERIAL calc-efs Trained.mtp_ testing.cfg testing_efs.cfg
 $MLP_COMMAND_SERIAL select-add Trained.mtp_ training.cfg testing.cfg diff.cfg > select.log
 cp training.cfg training_new.cfg 
 cat diff.cfg >> training_new.cfg
